/*
        Radino-WiFi ESP based webserver for In-Circuit radino WiFi modules on Spider
        for more information: www.in-circuit.de or www.radino.cc

	Copyright (c) 2015 In-Circuit GmbH

        v0.2 - 2015.03.02
	    Migration to In-Circuit ESP Library

        v0.1 - 2014.12.19
	    Basic implementation - functions and comments are subject to change

	Permission is hereby granted, free of charge, to any person obtaining a copy of this software
	and associated documentation files (the "Software"), to deal in the Software without restriction,
	including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
	LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
	IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
/*
  This example demonstrates how to build a communcation partner for the ESP webserver.
  All radino-WiFi modules are flashed with this sketch on delivery.
  
  The ESP will open an access point with ssid "RADINO-WiFi" and password "12345678".
  The IP adress of the ESP is 192.168.2.1
  
  The ESP will start an http server on port 80 and forward write and read requests to the Arduino.
  
  You can now access a simple website with your webbrowser, when entering your radino's IP adress.
  http://192.168.2.1/
  
  Further documentation is available on wiki.in-circuit.de
  
- radino WiFi setup:
  Arduino <---SPI---> SC16IS750 <---UART---> ESP8266
  
- Internal connections between Arduino(ATmega32U4),SC16IS750 and ESP8266:
  Arduino          ESP8266        SC16IS750
  
  A4               CH_PD          ---
  D4               RST            ---
  D9               GPIO0          ---
  ---              GPIO2          GPIO0
  
- Pin setup for ESP8266:
  Pin            normal operation        Firmware-Update-Mode
  
  CH_PD          HIGH                    HIGH
  GPIO0          HIGH                    LOW
  GPIO2          HIGH                    HIGH
  RST            HIGH                    HIGH
  
 */

// Use SPI-library for communication with ESP8266
#include <SPI.h>
#include <SPI_UART.h>
#include <IC_ESP.h>

IC_ESP esp = IC_ESP();

// Define Serial port that should be used for communication
// Valid values:
// USB-UART:       Serial
// Hardware-UART:  Serial1
#define dbgSerial Serial
#define DBG_SERIAL_BAUDRATE 57600

/* Registers */
unsigned char r1023 = 1234;  //counter
unsigned char r1024 = 0;  //led
const int ledPin = 13;

// Setup Serial connection and init ESP8266-Wifi-Module
void setup()
{
  dbgSerial.begin(DBG_SERIAL_BAUDRATE);  // Init Serial for status messages
  
  /*
  //Wait for USB serial ready, so all output can be seen on terminal
  // !! If USB Serial is not opened the module will wait here forever
  //Has no effect on hardware serial
  while(!dbgSerial);
  */
  
  delay(100);  // Wait 100 ms
  
  dbgSerial.println("Hello");  // Print something to show that Serial connection has established
  
  esp.setMode(ESP_IC_PROTOCOL_MODE);
  esp.init();  // Init ESP8266EX

  unsigned char cnt = 0;
  // Wait for ESP
  while(!esp.ready())
  {
    if(!(++cnt%10)) esp.init(); // Re-Init ESP
    dbgSerial.println("ESP !ready...");
    delay(500);
  }
  
  //Set Pins
  pinMode(A2, INPUT); // Potentiometer on Pin A2; analog input
  pinMode(ledPin, OUTPUT);  //LED
  
  dbgSerial.print("ESP library firmware version: ");
  dbgSerial.println(esp.getFirmwareVersion());
  
  /*
  //Starting as access point
  dbgSerial.print("Set SSID:");
  if(esp.AP_setSSID("RADINO-WiFi"))  dbgSerial.println("ok");
    else dbgSerial.println("er");

  dbgSerial.print("Set PW:");
  if(esp.AP_setPW("12345678"))  dbgSerial.println("ok");
    else dbgSerial.println("er");
  
  dbgSerial.print("Set Mode:");
  if(esp.NET_mode(ESP_WIFI_AP))  dbgSerial.println("ok");
    else dbgSerial.println("er");

  dbgSerial.print("starting Wifi:");
  if(esp.NET_wifiStart())  dbgSerial.println("ok");
    else dbgSerial.println("er");
  */
  
  /*
  //Starting in station mode
  dbgSerial.print("Set SSID:");
  if(esp.ST_setSSID("RADINO-WiFi"))  dbgSerial.println("ok");
    else dbgSerial.println("er");
  
  dbgSerial.print("Set PW:");
  if(esp.ST_setPW("12345678"))  dbgSerial.println("ok");
    else dbgSerial.println("er");
  
  dbgSerial.print("Set Mode:");
  if(esp.NET_mode(ESP_WIFI_STATION))  dbgSerial.println("ok");
    else dbgSerial.println("er");
  
  dbgSerial.print("starting Wifi:");
  if(esp.NET_wifiStart())  dbgSerial.println("ok");
    else dbgSerial.println("er");
  */
  
  //Wait until network established
  unsigned long startMillis = millis();
  unsigned long ip = 0;
  while(!ip) {
    ip = esp.getIPAddress();
    if (millis() - startMillis > 15000) {
      dbgSerial.println("no valid IP received for 15 Seconds");
      break;
    }
    delay(10);
  }
  //Print current IP
  dbgSerial.print("IP Address: ");
  dbgSerial.print((ip>>24)&0xFF);
  dbgSerial.print(".");
  dbgSerial.print((ip>>16)&0xFF);
  dbgSerial.print(".");
  dbgSerial.print((ip>>8)&0xFF);
  dbgSerial.print(".");
  dbgSerial.print((ip>>0)&0xFF);
  dbgSerial.println();
  
  /*
  //Set http port. ESP defaults to port 80
  dbgSerial.println("Setting HTTP-Port...");
  esp.HTTPD_setServerPort(81);
  */
  
  dbgSerial.println("Starting HTTP-Server...");
  esp.HTTPD_startServer();
  
  extern char * website;
  //Update Website from string
  dbgSerial.println("Updating Website...");
  esp.HTTPD_updateWebsite(website);
  dbgSerial.println("ok");
  delay(100);
  
  dbgSerial.println("Setup done");
}

// Wait for requests
void loop()
{
  unsigned short data = 0;
  
  /*
  // Upload website to ESP using Serial
  static unsigned long uartTimeout = 0;
  static char testBuf[9] = {0, };
  while (dbgSerial.available())
  {
    // Test if upload command started
    for(data=0;data<8;data++) testBuf[data]=testBuf[data+1];
    testBuf[8] = dbgSerial.read();
    if (strncmp(testBuf,"ICUPLOAD$",9)==0)
    {
      esp.disableInterrupt();
      delay(5);
      esp.write('I');esp.write('C');esp.write('U');esp.write('P');esp.write('L');esp.write('O');esp.write('A');esp.write('D');esp.write('$');
      uartTimeout = millis()+100;
      // Bridge data to ESP
      while(millis()<uartTimeout)
      {
        for ( data=dbgSerial.available() ; data || (data=dbgSerial.available()) ; data-- )
        {
          uartTimeout = millis()+100;
          esp.write(dbgSerial.read());
        }
        for ( data=esp.available() ; data || (data=esp.available()) ; data-- )
        {
          uartTimeout = millis()+100;
          dbgSerial.write(esp.read());
        }
      }
      delay(5);
      esp.enableInterrupt();
      delay(5);
    }
  }
  */
  
  //Push value every 2.5s to TCP server
  static unsigned long pushTimeout = 0;
  static unsigned char pushCounter = 0;
  if (millis()<pushTimeout)
  {
    esp.TCP_push(IP(192,168,2,2), 1022, String(analogRead(A2)*3.3/1024));
    pushTimeout = millis()+2500;
  }
  
  while(esp.newEventAvailable())
  {
    // new event arrived
    struct event e = esp.getEvent();
    
    switch (e.type) {
      case EVENT_HTTP_GET_REQUEST:
        switch(e.reg) {
            
          case 1022:
            data = analogRead(A2);
            esp.sendEventResponse(String(data*3.3/1024));
            break;
            
          case 1023:
            esp.sendEventResponse(r1023++);
            break;
            
          case 1024:
            esp.sendEventResponse(r1024);
            break;
            
          default:
            dbgSerial.println("unknown register");
            
        }
        break;
      case EVENT_HTTP_SET_REQUEST:
        switch(e.reg) {
            
          case 1023:
            r1023 = e.value;
            esp.sendEventResponse(r1023);
            break;
            
          case 1024:
            r1024 = e.value;
            esp.sendEventResponse(r1024);
            if (r1024) {
              digitalWrite(ledPin, HIGH);
            } else {
              digitalWrite(ledPin, LOW);
            }
            break;
            
          default:
            dbgSerial.println("unknown register");
            
        }
        break;
      default:
        dbgSerial.println("unknown event");
    }
  }
}

